<?php
/*
 * LibZFS.php
 *
 * part of pfSense (https://www.pfsense.org)
 * Copyright (c) 2013-2022 Rubicon Communications, LLC (Netgate)
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

namespace Netgate\Services\ZFS;

use mikehaertl\shellcommand\Command;

use Symfony\Contracts\Cache\ItemInterface;
use Symfony\Component\Cache\Adapter\AbstractAdapter;
use Symfony\Component\Cache\Adapter\PhpFilesAdapter;

// Pull in pfSense globals
require_once('globals.inc');

final class LibZFS {
	private const PYTHON_PATH = '/usr/local/bin/python3.8';

	private const ZFS_CACHE_KEY = 'ZFS';

	private $cache = null;

	public function __construct(AbstractAdapter $cache = null) {
		global $g;

		// Support dependency injection but also a default cache provider
		if (is_null($cache) || (!($cache instanceof AbstractAdapter))) {
			$cache = new PhpFilesAdapter(
				$namespace = $g['product_name'],
				$defaultLifetime = 30,
				$directory = "{$g['tmp_path']}/symfony-cache"
			);
		}

		$this->cache = $cache;

	}

	public function getCache() {
		return $this->cache;
	}

	public function setCache(AbstractAdapter $cache) {
		$this->_construct($cache);

		return $this;
	}

	public function flushCache() {
		$this->getCache()->delete(self::ZFS_CACHE_KEY);

		return $this;
	}

	public function getZFSData($flush = false) {
		if ($flush) {
			$this->flushCache();
		}

		return $this->getCache()->get(self::ZFS_CACHE_KEY, function (ItemInterface $item, &$save) {
			$cmd = new Command(self::PYTHON_PATH);

			$cmd->setStdIn($this->_getPythonScript());

			$cmd->execute();

			$save = ($cmd->getExitCode() === 0) && !empty($cmd->getOutput());

			return $save ? json_decode($cmd->getOutput(), true) : array();
		});
	}

	private function _getPythonScript() {
		return	<<<"PYTHON"

			import importlib
			import json
			import sys
			
			requiredModules = ['libzfs']
			
			for module in requiredModules:
				try:
					mod = importlib.import_module(module)
				except Exception:
					sys.exit(1)
				else:
					globals()[module] = mod
			
			try:
				zfs = libzfs.ZFS()
				zfsState = zfs.__getstate__()
				jsonOutput = json.dumps(zfsState, default=str)
			except Exception:
				sys.exit(1)
			else:
				print(jsonOutput)
				sys.exit(0)

			PYTHON;
	}
}

?>